""" XML Management.
    There are two XML file formats: Changelists and Workspace.
    Each format is handled by it's own package.
    Both Packages provide the same high level methods: read_xml and load_tree
"""
